@@ -49,11 +49,13 @@
 #include "handler_dirlist.h"
 #include "list_ext.h"
 
+#define ENTRIES "handler,common"
+
 
 cherokee_module_info_handler_t MODULE_INFO(common) = {
 	.module.type     = cherokee_handler,                /* type         */
 	.module.new_func = cherokee_handler_common_new,     /* new func     */
-	.valid_methods   = http_get | http_head             /* http methods */
+	.valid_methods   = http_all_methods                 /* http methods */
 };
 
 
@@ -67,7 +69,10 @@ stat_file (cherokee_boolean_t useit, cherokee_iocache_t *iocache, struct stat *n
 	/* Without cache
 	 */
 	if (!useit) {
-		re = stat (path, nocache_info);
+		re = cherokee_stat (path, nocache_info);
+
+		TRACE (ENTRIES, "%s, use_iocache=%d re=%d\n", path, useit, re);
+
 		if (re < 0) {
 			switch (errno) {
 			case ENOENT: 
@@ -85,7 +90,11 @@ stat_file (cherokee_boolean_t useit, cherokee_iocache_t *iocache, struct stat *n
 
 	/* I/O cache
 	 */
+#ifndef CHEROKEE_EMBEDDED
 	ret = cherokee_iocache_stat_get (iocache, path, io_entry);
+
+	TRACE (ENTRIES, "%s, use_iocache=%d re=%d\n", path, useit, re);
+
 	if (ret != ret_ok) {
 		switch (ret) {
 		case ret_not_found:
@@ -99,6 +108,9 @@ stat_file (cherokee_boolean_t useit, cherokee_iocache_t *iocache, struct stat *n
 
 	*info = &(*io_entry)->state;
 	return ret_ok;
+#endif
+
+	return ret_error;
 }
 
 
@@ -109,16 +121,11 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 	int                       exists;
 	struct stat               nocache_info;
 	struct stat              *info;
-	cherokee_iocache_t       *iocache;
 	cherokee_iocache_entry_t *file;
+	cherokee_iocache_t       *iocache     = NULL;
  	cherokee_boolean_t        use_iocache = true;
 	cherokee_connection_t    *conn        = CONN(cnt);
 
-#if 0
-	PRINT_DEBUG ("request: '%s'\n", conn->request->buf);
-	PRINT_DEBUG ("local:   '%s'\n", conn->local_directory->buf);
-#endif
-
 	/* Check some properties
 	 */
 	if (properties != NULL) {
@@ -129,9 +136,17 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 	 */
 	cherokee_buffer_add_buffer (&conn->local_directory, &conn->request);
 
+#ifdef CHEROKEE_EMBEDDED
+	use_iocache = false;
+#else
 	cherokee_iocache_get_default (&iocache);
+#endif
+
 	ret = stat_file (use_iocache, iocache, &nocache_info, conn->local_directory.buf, &file, &info);
 	exists = (ret == ret_ok);
+
+	TRACE (ENTRIES, "request: '%s', local: '%s', exists %d\n", 
+	       conn->request.buf, conn->local_directory.buf, exists);
 	
 	if (!exists) {
 		ret_t  ret;
@@ -144,7 +159,7 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 		 */
 		begin = conn->local_directory.len - conn->request.len;
 		
-		ret = cherokee_split_pathinfo (&conn->local_directory, begin, &pathinfo, &pathinfo_len);
+		ret = cherokee_split_pathinfo (&conn->local_directory, begin, true, &pathinfo, &pathinfo_len);
 		if ((ret == ret_not_found) || (pathinfo_len <= 0)) {
 			conn->error_code = http_not_found;
 			return ret_error;
@@ -167,6 +182,7 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 	/* Is it a file?
 	 */
 	if (S_ISREG(info->st_mode)) {
+		TRACE (ENTRIES, "going for %s\n", "handler_file");
 		return cherokee_handler_file_new (hdl, cnt, properties);
 	}
 
@@ -178,6 +194,7 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 		/* Maybe it has to be redirected
 		 */
 		if (conn->request.buf[conn->request.len-1] != '/') {
+			TRACE (ENTRIES, "going for %s\n", "handler_dir");
 			return cherokee_handler_dirlist_new (hdl, cnt, properties);
 		}
 
@@ -222,6 +239,8 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 				cherokee_buffer_clean (&conn->request);
 				cherokee_buffer_add (&conn->request, index, index_len);				
 
+				TRACE (ENTRIES, "top level index matched %s\n", index);
+
 				return ret_eagain;
 			}
 
@@ -232,6 +251,8 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 			ret = stat_file (use_iocache, iocache, &nocache_info, conn->local_directory.buf, &file, &info);
 			exists = (ret == ret_ok);
 
+			TRACE (ENTRIES, "trying index '%s', exists %d\n", index, exists);
+
 			cherokee_buffer_drop_endding (&conn->local_directory, index_len);
 
 			/* If the file doesn't exist or it is a directory, try with the next one
