@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001, 2002, 2003, 2004, 2005 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2006 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -28,7 +28,7 @@
 ret_t 
 cherokee_module_init_base  (cherokee_module_t *module)
 {
-	   module->new      = NULL;
+	   module->instance = NULL;
 	   module->init     = NULL;
 	   module->get_name = NULL;
 	   module->free     = NULL;
