@@ -94,6 +94,59 @@ extern int32_t sendfile (int out_fd, int in_fd, int32_t *offset, uint32_t count)
 #include "virtual_server.h"
 
 
+ret_t
+cherokee_socket_init (cherokee_socket_t *socket)
+{
+	memset (&socket->client_addr, 0, sizeof(cherokee_sockaddr_t));
+	socket->client_addr_len = -1;
+
+	socket->socket      = -1;
+	socket->status      = socket_closed;
+	socket->is_tls      = non_TLS;
+
+#ifdef HAVE_TLS
+	socket->initialized = false;
+	socket->vserver_ref = NULL;
+#endif
+
+#ifdef HAVE_GNUTLS
+	socket->session     = NULL;
+#endif
+
+#ifdef HAVE_OPENSSL
+	socket->session     = NULL;
+	socket->ssl_ctx     = NULL;
+#endif
+
+	return ret_ok;
+}
+
+
+ret_t
+cherokee_socket_mrproper (cherokee_socket_t *socket)
+{
+#ifdef HAVE_GNUTLS
+	if (socket->session != NULL) {
+		gnutls_deinit (socket->session);
+		socket->session = NULL;
+	}
+#endif
+
+#ifdef HAVE_OPENSSL
+	if (socket->session != NULL) {
+		SSL_free (socket->session);
+		socket->session = NULL;
+	}
+
+	if (socket->ssl_ctx != NULL) {
+		SSL_CTX_free (socket->ssl_ctx);
+		socket->ssl_ctx = NULL;
+	}
+#endif
+
+	return ret_ok;
+}
+
 ret_t
 cherokee_socket_new (cherokee_socket_t **socket)
 {
@@ -101,19 +154,8 @@ cherokee_socket_new (cherokee_socket_t **socket)
 
 	/* Init 
 	 */
-#ifdef HAVE_TLS
-	n->session = NULL;
-# ifdef HAVE_OPENSSL
-	n->ssl_ctx = NULL;
-# endif
-#endif
-	cherokee_socket_clean (n);
-
-	/* Client address
-	 */
-	memset (&n->client_addr, 0, sizeof(struct sockaddr));
-	n->client_addr_len = -1;
-
+	cherokee_socket_init (n);
+	
 	/* Return it
 	 */
 	*socket = n;
@@ -330,7 +372,7 @@ ret_t
 cherokee_socket_init_tls (cherokee_socket_t *socket, cherokee_virtual_server_t *vserver)
 {
 #ifdef HAVE_TLS
-	int rc;
+	int re;
 	
 	socket->is_tls = TLS;
 
@@ -340,28 +382,28 @@ cherokee_socket_init_tls (cherokee_socket_t *socket, cherokee_virtual_server_t *
 	}
 
 # ifdef HAVE_GNUTLS
-	rc = gnutls_handshake (socket->session);
+	re = gnutls_handshake (socket->session);
 
-	switch (rc) {
+	switch (re) {
 	case GNUTLS_E_AGAIN:
 		return ret_eagain;
 	case GNUTLS_E_INTERRUPTED:
 		return ret_error;
 	}
 	
-	if (rc < 0) {
-		PRINT_ERROR ("ERROR: Init GNUTLS: Handshake has failed: %s\n", gnutls_strerror(rc));
+	if (re < 0) {
+		PRINT_ERROR ("ERROR: Init GNUTLS: Handshake has failed: %s\n", gnutls_strerror(re));
 		return ret_error;
 	}
 # endif
 
 # ifdef HAVE_OPENSSL
-	rc = SSL_accept (socket->session);
+	re = SSL_accept (socket->session);
 
-	if (rc <= 0) {
+	if (re <= 0) {
 		char *error;
 
-		switch (SSL_get_error(socket->session, rc)) {
+		switch (SSL_get_error(socket->session, re)) {
 		case SSL_ERROR_WANT_READ:
 		case SSL_ERROR_WANT_WRITE:
 		case SSL_ERROR_WANT_CONNECT:
@@ -381,24 +423,7 @@ cherokee_socket_init_tls (cherokee_socket_t *socket, cherokee_virtual_server_t *
 ret_t
 cherokee_socket_free (cherokee_socket_t *socket)
 {
-#ifdef HAVE_GNUTLS
-	if (socket->session != NULL) {
-		gnutls_deinit (socket->session);
-		socket->session = NULL;
-	}
-#endif
-
-#ifdef HAVE_OPENSSL
-	if (socket->session != NULL) {
-		SSL_free (socket->session);
-		socket->session = NULL;
-	}
-
-	if (socket->ssl_ctx != NULL) {
-		SSL_CTX_free (socket->ssl_ctx);
-		socket->ssl_ctx = NULL;
-	}
-#endif
+	cherokee_socket_mrproper (socket);
 
 	free (socket);
 	return ret_ok;
@@ -409,7 +434,7 @@ cherokee_socket_close (cherokee_socket_t *socket)
 {
 	int re;
 
-	if (socket->socket <= 0) {
+	if (socket->socket < 0) {
 		return ret_error;
 	}
 
@@ -446,7 +471,7 @@ cherokee_socket_shutdown (cherokee_socket_t *socket, int how)
 {
 	int re;
 
-	if (socket->socket <= 0) {
+	if (socket->socket < 0) {
 		return ret_error;
 	}
 
@@ -463,7 +488,7 @@ cherokee_socket_ntop (cherokee_socket_t *socket, char *dst, size_t cnt)
 
 	errno = EAFNOSUPPORT;
 
-	if (SOCKET_FD(socket) <= 0) {
+	if (SOCKET_FD(socket) < 0) {
 		return ret_error;
 	}
 
@@ -577,7 +602,7 @@ cherokee_socket_accept_fd (int server_socket, int *new_fd, cherokee_sockaddr_t *
 	 */
 	len = sizeof (cherokee_sockaddr_t);
 	new_socket = accept (server_socket, &sa->sa, &len);
-	if (new_socket <= 0) {
+	if (new_socket < 0) {
 		return ret_error;
 	}		
 
@@ -614,19 +639,6 @@ cherokee_socket_set_client (cherokee_socket_t *sock, int type)
 }
 
 
-ret_t       
-cherokee_socket_set_status (cherokee_socket_t *socket, cherokee_socket_status_t status)
-{
-	socket->status = status;
-
-	if (status == socket_closed) {
-		cherokee_socket_close (socket);
-	}
-
-	return ret_ok;
-}
-
-
 ret_t 
 cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len, size_t *written)
 {
@@ -644,6 +656,7 @@ cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len, size_t
 			case GNUTLS_E_PUSH_ERROR:
 			case GNUTLS_E_INTERRUPTED:
 			case GNUTLS_E_INVALID_SESSION: 
+				socket->status = socket_closed;
 				return ret_eof;
 			case GNUTLS_E_AGAIN:           
 				return ret_eagain;
@@ -651,11 +664,10 @@ cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len, size_t
 
 			PRINT_ERROR ("ERROR: GNUTLS: gnutls_record_send(%d, ..) -> err=%d '%s'\n", 
 				     SOCKET_FD(socket), (int)len, gnutls_strerror(len));
-
-			*written = 0;
 			return ret_error;
 
 		} else if (len == 0) {
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 #endif
@@ -674,11 +686,10 @@ cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len, size_t
 
 			PRINT_ERROR ("ERROR: SSL_write (%d, ..) -> err=%d '%s'\n", 
 				     SOCKET_FD(socket), len, ERR_error_string(re, NULL));
-
-			*written = 0;
 			return ret_error;
 
 		} else if (len == 0) {
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 #endif		
@@ -701,16 +712,16 @@ cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len, size_t
 		case ETIMEDOUT:
 		case ECONNRESET:  
 		case EHOSTUNREACH:
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 	
 		PRINT_ERROR ("ERROR: write(%d, ..) -> errno=%d '%s'\n", 
 			     SOCKET_FD(socket), err, strerror(err));
-
-		*written = 0;
 		return ret_error;
 
 	}  else if (len == 0) {
+		socket->status = socket_closed;
 		return ret_eof;
 	}
 
@@ -727,21 +738,16 @@ cherokee_read (cherokee_socket_t *socket, char *buf, int buf_size, size_t *done)
 {
 	ssize_t len;
 
-	if (socket->is_tls == TLS) {
+	if ((socket->is_tls == TLS) && (buf != NULL)) {
 #ifdef HAVE_GNUTLS
-		if (unlikely (buf == NULL)) {
-			char tmp[buf_size+1];
-			len = gnutls_record_recv (socket->session, tmp, buf_size);
-		} else {
-			len = gnutls_record_recv (socket->session, buf, buf_size);
-		}
-
+		len = gnutls_record_recv (socket->session, buf, buf_size);
 		if (len < 0) {
 			switch (len) {
 			case GNUTLS_E_PUSH_ERROR:
 			case GNUTLS_E_INTERRUPTED:              
 			case GNUTLS_E_INVALID_SESSION:
 			case GNUTLS_E_UNEXPECTED_PACKET_LENGTH:
+				socket->status = socket_closed;
 				return ret_eof;
 			case GNUTLS_E_AGAIN:
 				return ret_eagain;
@@ -749,40 +755,36 @@ cherokee_read (cherokee_socket_t *socket, char *buf, int buf_size, size_t *done)
 			
 			PRINT_ERROR ("ERROR: GNUTLS: gnutls_record_recv(%d, ..) -> err=%d '%s'\n", 
 				     SOCKET_FD(socket), (int)len, gnutls_strerror(len));
-
-			*done = 0;
 			return ret_error;
 			
 		} else if (len == 0) {
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 #endif
 
 #ifdef HAVE_OPENSSL
-		if (unlikely (buf == NULL)) {
-			char tmp[buf_size+1];
-			len = SSL_read (socket->session, tmp, buf_size);
-		} else {
-			len = SSL_read (socket->session, buf, buf_size);
-		}
-		
+		len = SSL_read (socket->session, buf, buf_size);
 		if (len < 0) {
 			int re;
 
 			re = SSL_get_error (socket->session, len);
 			switch (re) {
-			case SSL_ERROR_WANT_READ:   return ret_eagain;
-			case SSL_ERROR_ZERO_RETURN: return ret_eof;
-			case SSL_ERROR_SSL:         return ret_error;
+			case SSL_ERROR_WANT_READ:   
+				return ret_eagain;
+			case SSL_ERROR_ZERO_RETURN: 
+				socket->status = socket_closed;
+				return ret_eof;
+			case SSL_ERROR_SSL:         
+				return ret_error;
 			}
 
 			PRINT_ERROR ("ERROR: OpenSSL: SSL_read (%d, ..) -> err=%d '%s'\n", 
 				     SOCKET_FD(socket), len, ERR_error_string(re, NULL));
-
-			*done = 0;
 			return ret_error;
 
 		} else if (len == 0) {
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 #endif
@@ -792,8 +794,8 @@ cherokee_read (cherokee_socket_t *socket, char *buf, int buf_size, size_t *done)
 	/* Plain read
 	 */
 	if (unlikely (buf == NULL)) {
-		char tmp[buf_size+1];
-		len = recv (SOCKET_FD(socket), tmp, buf_size, 0);
+		static char trash[256];
+		len = recv (SOCKET_FD(socket), trash, 256, 0);
 	} else {
 		len = recv (SOCKET_FD(socket), buf, buf_size, 0);
 	}
@@ -814,16 +816,16 @@ cherokee_read (cherokee_socket_t *socket, char *buf, int buf_size, size_t *done)
 		case ETIMEDOUT:
 		case ECONNRESET:
 		case EHOSTUNREACH:
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 
 		PRINT_ERROR ("ERROR: read(%d, ..) -> errno=%d '%s'\n", 
 			     SOCKET_FD(socket), err, strerror(err));
-		
-		*done = 0;
 		return ret_error;
 
 	} else if (len == 0) {
+		socket->status = socket_closed;
 		return ret_eof;
 	}
 	
@@ -871,14 +873,12 @@ cherokee_writev (cherokee_socket_t *socket, const struct iovec *vector, uint16_t
 		case EPIPE:
 		case ETIMEDOUT:
 		case ECONNRESET:
+			socket->status = socket_closed;
 			return ret_eof;
 		}
 	       
 		PRINT_ERROR ("ERROR: writev(%d, ..) -> errno=%d '%s'\n", 
 			     SOCKET_FD(socket), err, strerror(err));
-
-		
-		*written = 0;
 		return ret_error;
 	}
 	
@@ -1059,7 +1059,7 @@ ret_t
 cherokee_socket_gethostbyname (cherokee_socket_t *socket, cherokee_buffer_t *hostname)
 {
 	if (SOCKET_AF(socket) == AF_UNIX) {
-#ifndef _WIN32
+#ifdef _WIN32
 		SHOULDNT_HAPPEN;
 		return ret_no_sys;
 #else
@@ -1080,14 +1080,18 @@ cherokee_socket_connect (cherokee_socket_t *socket)
 	int r;
 
 	if (SOCKET_AF(socket) == AF_UNIX) {
-#ifdef _WIN32
+#ifndef _WIN32
+		r = connect (SOCKET_FD(socket), (struct sockaddr *) &SOCKET_ADDR_UNIX(socket), sizeof(SOCKET_ADDR_UNIX(socket)));
+#else
 		SHOULDNT_HAPPEN;
 		return ret_no_sys;
-#else
-		r = connect (SOCKET_FD(socket), (struct sockaddr *) &SOCKET_ADDR_UNIX(socket), sizeof(SOCKET_ADDR_UNIX(socket)));
 #endif
 	} else {
-		r = connect (SOCKET_FD(socket), (struct sockaddr *) &SOCKET_ADDR(socket), sizeof(cherokee_sockaddr_t));
+#ifdef HAVE_IPV6
+		r = connect (SOCKET_FD(socket), (struct sockaddr *) &SOCKET_ADDR(socket), sizeof(struct sockaddr_in6));
+#else
+		r = connect (SOCKET_FD(socket), (struct sockaddr *) &SOCKET_ADDR(socket), sizeof(struct sockaddr_in));
+#endif
 	}
 
 	if (r < 0) {
@@ -1109,6 +1113,7 @@ cherokee_socket_connect (cherokee_socket_t *socket)
 		}
 	}
 
+	socket->status = socket_reading;
 	return ret_ok;
 }
 
@@ -1224,7 +1229,7 @@ cherokee_socket_set_timeout (cherokee_socket_t *socket, cuint_t timeout)
 	cuint_t        block;
 	struct timeval tv;
 
-	if (socket->socket <= 0) {
+	if (socket->socket < 0) {
 		return ret_error;
 	}
 
@@ -1239,24 +1244,35 @@ cherokee_socket_set_timeout (cherokee_socket_t *socket, cuint_t timeout)
 	re = ioctlsocket (socket->socket, FIONBIO, &block);	
 	if (re == SOCKET_ERROR) {
 #else	
-		re = ioctl (socket->socket, FIONBIO, &block);	
-		if (re < 0) {
+	re = ioctl (socket->socket, FIONBIO, &block);	
+	if (re < 0) {
 #endif
-			PRINT_ERROR ("ioctl (%d, FIONBIO, &%d) = %d\n", socket->socket, block, re);
-			return ret_error;
-		}
-
-		/* Set the send / receive timeouts
-		 */
-#ifdef SO_RCVTIMEO
-		tv.tv_sec  = timeout / 1000;
-		tv.tv_usec = timeout % 1000;
-
-		re = setsockopt (socket->socket, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
-		if (re < 0) {
-			PRINT_ERROR ("Couldn't set SO_RCVTIMEO, fd=%d, timeout=%d\n", socket->socket, timeout);
-		}
-#endif
-
-		return ret_ok;		      
+		PRINT_ERROR ("ioctl (%d, FIONBIO, &%d) = %d\n", socket->socket, block, re);
+		return ret_error;
 	}
+
+	/* Set the send / receive timeouts
+	 */
+#if defined(SO_RCVTIMEO) && !defined(HAVE_BROKEN_SO_RCVTIMEO)
+	tv.tv_sec  = timeout / 1000;
+	tv.tv_usec = timeout % 1000;
+	
+	re = setsockopt (socket->socket, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
+	if (re < 0) {
+		int err = errno;
+		PRINT_ERROR ("Couldn't set SO_RCVTIMEO, fd=%d, timeout=%d: %s\n", 
+			     socket->socket, timeout, strerror(err));
+	}
+#endif
+	
+	return ret_ok;		      
+}
+
+
+ret_t       
+cherokee_socket_set_status (cherokee_socket_t *socket, cherokee_socket_status_t status)
+{
+	socket->status = status;
+	return ret_ok;
+}
+
